@@ -408,6 +408,7 @@ std::string_view ROStreamBuf::getStringView( const size_t size /* = 0 */ )
 bool StreamFile::open( const std::string & fn, const std::string & mode )
 {
     _file.reset( std::fopen( fn.c_str(), mode.c_str() ) );
+    // codechecker_false_positive [alpha.unix.Stream] Opened stream never closed. Potential resource leak
     if ( !_file ) {
         ERROR_LOG( "Error opening file " << fn )
     }
@@ -589,8 +590,12 @@ void StreamFile::putLE32( uint32_t v )
 
 std::vector<uint8_t> StreamFile::getRaw( const size_t size )
 {
+    if ( !_file ) {
+        return {};
+    }
+
     const size_t chunkSize = size > 0 ? size : sizeg();
-    if ( chunkSize == 0 || !_file ) {
+    if ( chunkSize == 0 ) {
         return {};
     }
 
