@@ -49,12 +49,14 @@ namespace AI
         int spellID{ -1 };
         int32_t cell{ -1 };
         double value{ 0.0 };
+        int32_t destinationCell{ -1 };
     };
 
     struct SpellcastOutcome
     {
         int32_t cell{ -1 };
         double value{ 0.0 };
+        int32_t destinationCell{ -1 };
 
         void updateOutcome( const double potentialValue, const int32_t targetCell, const bool isMassEffect = false )
         {
@@ -96,7 +98,7 @@ namespace AI
 
         bool isPositionLocatedInDefendedArea( const Battle::Unit & currentUnit, const Battle::Position & pos ) const;
 
-        SpellSelection selectBestSpell( Battle::Arena & arena, const Battle::Unit & currentUnit, bool retreating ) const;
+        SpellSelection selectBestSpell( Battle::Arena & arena, const Battle::Unit & currentUnit, const bool retreating ) const;
 
         SpellcastOutcome spellDamageValue( const Spell & spell, Battle::Arena & arena, const Battle::Unit & currentUnit, const Battle::Units & friendly,
                                            const Battle::Units & enemies, bool retreating ) const;
@@ -104,14 +106,20 @@ namespace AI
         SpellcastOutcome spellResurrectValue( const Spell & spell, const Battle::Arena & arena ) const;
         SpellcastOutcome spellSummonValue( const Spell & spell, const Battle::Arena & arena, const PlayerColor heroColor ) const;
         SpellcastOutcome spellDragonSlayerValue( const Spell & spell, const Battle::Units & friendly, const Battle::Units & enemies ) const;
-        SpellcastOutcome spellEffectValue( const Spell & spell, const Battle::Units & targets ) const;
+        SpellcastOutcome spellTeleportValue( Battle::Arena & arena, const Spell & spell, const Battle::Unit & currentUnit, const Battle::Units & enemies ) const;
+        SpellcastOutcome spellEarthquakeValue( const Battle::Arena & arena, const Spell & spell, const Battle::Units & friendly ) const;
+        SpellcastOutcome spellEffectValue( const Spell & spell, const Battle::Units & targets, const Battle::Units & enemies ) const;
 
-        double spellEffectValue( const Spell & spell, const Battle::Unit & target, bool targetIsLast, bool forDispel ) const;
+        double spellEffectValue( const Spell & spell, const Battle::Unit & target, const Battle::Units & enemies, const bool targetIsLast, const bool forDispel ) const;
         double getSpellDisruptingRayRatio( const Battle::Unit & target ) const;
         double getSpellSlowRatio( const Battle::Unit & target ) const;
         double getSpellHasteRatio( const Battle::Unit & target ) const;
         int32_t spellDurationMultiplier( const Battle::Unit & target ) const;
 
+        bool isSpellcastUselessForUnit( const Battle::Unit & unit, const Battle::Units & enemies, const Spell & spell ) const;
+
+        static double getMeleeBestOutcome( Battle::Arena & arena, const Battle::Unit & currentUnit, const Battle::Units & enemies, BattleTargetPair & bestTarget );
+
         // When this limit of turns without deaths is exceeded for an attacking AI-controlled hero,
         // the auto combat should be interrupted (one way or another)
         static const uint32_t MAX_TURNS_WITHOUT_DEATHS{ 50 };
@@ -119,8 +127,8 @@ namespace AI
         // Member variables related to the logic of checking the limit of the number of turns
         uint32_t _currentTurnNumber{ 0 };
         uint32_t _numberOfRemainingTurnsWithoutDeaths{ MAX_TURNS_WITHOUT_DEATHS };
-        uint32_t _attackerForceNumberOfDead{ 0 };
-        uint32_t _defenderForceNumberOfDead{ 0 };
+        uint32_t _attackerForceTotalNumberOfDeadUnits{ 0 };
+        uint32_t _defenderForceTotalNumberOfDeadUnits{ 0 };
 
         // Member variables with a lifetime in one turn
         const HeroBase * _commander{ nullptr };
