@@ -27,6 +27,7 @@
 #include <array>
 #include <cassert>
 #include <cstddef>
+#include <initializer_list>
 #include <iterator>
 #include <numeric>
 #include <utility>
@@ -38,6 +39,7 @@
 #include "cursor.h"
 #include "dialog.h"
 #include "game_hotkeys.h"
+#include "game_string.h"
 #include "ground.h"
 #include "heroes_base.h"
 #include "icn.h"
@@ -59,7 +61,6 @@
 #include "ui_constants.h"
 #include "ui_dialog.h"
 #include "ui_map_object.h"
-#include "ui_scrollbar.h"
 #include "ui_text.h"
 #include "ui_tool.h"
 #include "world.h"
@@ -151,6 +152,8 @@ namespace
             SetListContent( _ids );
             // For multi-selection we don't have any current item.
             SetCurrent( 0 );
+
+            enableToggleButtons();
         }
 
         std::vector<int> getSelected() const
@@ -168,6 +171,25 @@ namespace
             updateStatus( id );
         }
 
+    protected:
+        void onToggleOn() override
+        {
+            _selected = {};
+
+            for ( const int id : _ids ) {
+                _selected.emplace( id );
+            }
+
+            setButtonOkayStatus( true );
+        }
+
+        void onToggleOff() override
+        {
+            _selected = {};
+
+            setButtonOkayStatus( false );
+        }
+
     private:
         std::vector<int> _ids;
         std::set<int> _selected;
@@ -648,6 +670,11 @@ namespace
 
     int selectObjectType( const int objectType, const size_t objectCount, ObjectTypeSelection & objectSelection )
     {
+        if ( objectCount == 0 ) {
+            fheroes2::showStandardTextMessage( _( "Warning" ), _( "There is nothing to select from." ), Dialog::OK );
+            return -1;
+        }
+
         std::vector<int> objects( objectCount, 0 );
         std::iota( objects.begin(), objects.end(), 0 );
         objectSelection.SetListContent( objects );
@@ -655,7 +682,7 @@ namespace
         objectSelection.SetCurrent( std::max( objectType, 0 ) );
 
         const int32_t result = objectSelection.selectItemsEventProcessing();
-        return result == Dialog::OK ? objectSelection.GetCurrent() : -1;
+        return result == Dialog::OK && objectSelection.IsValid() ? objectSelection.GetCurrent() : -1;
     }
 
     class MineTypeList final : public Interface::ListBox<Maps::ObjectInfo>
@@ -677,11 +704,6 @@ namespace
             fheroes2::showStandardTextMessage( MP2::StringObject( info.objectType ), "", Dialog::ZERO );
         }
 
-        int getCurrentId() const
-        {
-            return _currentId;
-        }
-
         void initListBackgroundRestorer( fheroes2::Rect roi )
         {
             _listBackground = std::make_unique<fheroes2::ImageRestorer>( fheroes2::Display::instance(), roi.x, roi.y, roi.width, roi.height );
@@ -692,15 +714,6 @@ namespace
             return _isDoubleClicked;
         }
 
-        void updateScrollBarImage()
-        {
-            const int32_t scrollBarWidth = _scrollbar.width();
-
-            setScrollBarImage( fheroes2::generateScrollbarSlider( _scrollbar, false, _scrollbar.getArea().height, VisibleItemCount(), _size(),
-                                                                  { 0, 0, scrollBarWidth, 8 }, { 0, 7, scrollBarWidth, 8 } ) );
-            _scrollbar.moveToIndex( _topId );
-        }
-
         void RedrawItem( const Maps::ObjectInfo & info, int32_t dstx, int32_t dsty, bool current ) override
         {
             const fheroes2::Sprite mineSprite( fheroes2::generateMapObjectImage( info ) );
@@ -767,6 +780,11 @@ int32_t Dialog::selectKingdomCastle( const Kingdom & kingdom, const bool notOccu
         castles.push_back( castle->GetIndex() );
     }
 
+    if ( castles.empty() ) {
+        fheroes2::showStandardTextMessage( _( "Warning" ), _( "There are no castles to select from." ), Dialog::OK );
+        return -1;
+    }
+
     const int32_t maxHeight = std::min( 100 + SelectKingdomCastle::itemsOffsetY * 12, fheroes2::Display::instance().height() - dialogHeightDeduction );
     const int32_t itemsHeight = std::max( 100 + SelectKingdomCastle::itemsOffsetY * static_cast<int32_t>( castles.size() ), 100 + SelectKingdomCastle::itemsOffsetY * 5 );
     const int32_t totalHeight = std::min( itemsHeight, maxHeight );
@@ -780,7 +798,7 @@ int32_t Dialog::selectKingdomCastle( const Kingdom & kingdom, const bool notOccu
 
     const int32_t result = listbox.selectItemsEventProcessing();
 
-    return ( result == Dialog::OK ) ? listbox.GetCurrent() : -1;
+    return ( result == Dialog::OK && listbox.IsValid() ) ? listbox.GetCurrent() : -1;
 }
 
 namespace Dialog
@@ -831,16 +849,6 @@ namespace Dialog
         _window->renderOkayCancelButtons( _buttonOk, _buttonCancel );
     }
 
-    void ItemSelectionWindow::updateScrollBarImage()
-    {
-        const int32_t scrollBarWidth = _scrollbar.width();
-
-        setScrollBarImage( fheroes2::generateScrollbarSlider( _scrollbar, false, _scrollbar.getArea().height, VisibleItemCount(), _size(), { 0, 0, scrollBarWidth, 8 },
-                                                              { 0, 7, scrollBarWidth, 8 } ) );
-        _scrollbar.moveToIndex( _topId );
-    }
-
-    // An image with text should have offset of 10 pixels from all left and right edges.
     void ItemSelectionWindow::renderItem( const fheroes2::Sprite & itemSprite, std::string itemText, const fheroes2::Point & destination,
                                           const int32_t middleImageOffsetX, const int32_t textOffsetX, const int32_t itemOffsetY, const bool current ) const
     {
@@ -849,8 +857,28 @@ namespace Dialog
         fheroes2::Blit( itemSprite, display, destination.x + middleImageOffsetX - ( itemSprite.width() / 2 ), destination.y + itemOffsetY - ( itemSprite.height() / 2 ) );
 
         fheroes2::Text text( std::move( itemText ), current ? fheroes2::FontType::normalYellow() : fheroes2::FontType::normalWhite() );
+        renderText( text, destination, textOffsetX, itemOffsetY );
+    }
+
+    void ItemSelectionWindow::renderItem( const fheroes2::Sprite & itemSprite, std::vector<fheroes2::LocalizedString> itemText, const fheroes2::Point & destination,
+                                          const int32_t middleImageOffsetX, const int32_t textOffsetX, const int32_t itemOffsetY, const bool current ) const
+    {
+        fheroes2::Display & display = fheroes2::Display::instance();
+
+        fheroes2::Blit( itemSprite, display, destination.x + middleImageOffsetX - ( itemSprite.width() / 2 ), destination.y + itemOffsetY - ( itemSprite.height() / 2 ) );
+
+        auto text = fheroes2::getLocalizedText( std::move( itemText ), current ? fheroes2::FontType::normalYellow() : fheroes2::FontType::normalWhite() );
+        renderText( *text, destination, textOffsetX, itemOffsetY );
+    }
+
+    void ItemSelectionWindow::renderText( fheroes2::TextBase & text, const fheroes2::Point & destination, const int32_t textOffsetX, const int32_t itemOffsetY ) const
+    {
+        fheroes2::Display & display = fheroes2::Display::instance();
+
         text.fitToOneRow( _window->activeArea().width - textOffsetX - 55 );
-        text.draw( destination.x + textOffsetX, destination.y + itemOffsetY - ( text.height() / 2 ) + 2, display );
+        const fheroes2::Rect & roi = _window->activeArea();
+        text.drawInRoi( destination.x + textOffsetX, destination.y + itemOffsetY - ( text.height() / 2 ) + 2, display,
+                        { destination.x + textOffsetX, roi.y, roi.width - textOffsetX - 55, roi.height } );
     }
 
     int32_t ItemSelectionWindow::selectItemsEventProcessing()
@@ -873,6 +901,13 @@ namespace Dialog
             _buttonOk.drawOnState( le.isMouseLeftButtonPressedAndHeldInArea( _buttonOk.area() ) );
             _buttonCancel.drawOnState( le.isMouseLeftButtonPressedAndHeldInArea( _buttonCancel.area() ) );
 
+            if ( _buttonToggleOn.isEnabled() ) {
+                _buttonToggleOn.drawOnState( le.isMouseLeftButtonPressedAndHeldInArea( _buttonToggleOn.area() ) );
+            }
+            else if ( _buttonToggleOff.isEnabled() ) {
+                _buttonToggleOff.drawOnState( le.isMouseLeftButtonPressedAndHeldInArea( _buttonToggleOff.area() ) );
+            }
+
             if ( _buttonOk.isEnabled() && ( le.MouseClickLeft( _buttonOk.area() ) || Game::HotKeyPressEvent( Game::HotKeyEvent::DEFAULT_OKAY ) ) ) {
                 return Dialog::OK;
             }
@@ -880,12 +915,37 @@ namespace Dialog
                 return Dialog::CANCEL;
             }
 
+            if ( _buttonToggleOn.isEnabled() && le.MouseClickLeft( _buttonToggleOn.area() ) ) {
+                needRedraw = true;
+
+                onToggleOn();
+
+                _buttonToggleOn.disable();
+                _buttonToggleOff.enable();
+                _buttonToggleOff.draw();
+            }
+            else if ( _buttonToggleOff.isEnabled() && le.MouseClickLeft( _buttonToggleOff.area() ) ) {
+                needRedraw = true;
+
+                onToggleOff();
+
+                _buttonToggleOff.disable();
+                _buttonToggleOn.enable();
+                _buttonToggleOn.draw();
+            }
+
             if ( le.isMouseRightButtonPressedInArea( _buttonOk.area() ) ) {
                 fheroes2::showStandardTextMessage( _( "Okay" ), _( "Accept the choice made." ), Dialog::ZERO );
             }
             else if ( le.isMouseRightButtonPressedInArea( _buttonCancel.area() ) ) {
                 fheroes2::showStandardTextMessage( _( "Cancel" ), _( "Exit this menu without doing anything." ), Dialog::ZERO );
             }
+            else if ( _buttonToggleOn.isEnabled() && le.isMouseRightButtonPressedInArea( _buttonToggleOn.area() ) ) {
+                fheroes2::showStandardTextMessage( _( "Enable All Items" ), _( "Click to enable all items." ), Dialog::ZERO );
+            }
+            else if ( _buttonToggleOff.isEnabled() && le.isMouseRightButtonPressedInArea( _buttonToggleOff.area() ) ) {
+                fheroes2::showStandardTextMessage( _( "Disable All Items" ), _( "Click to disable all items." ), Dialog::ZERO );
+            }
 
             QueueEventProcessing();
 
@@ -909,6 +969,21 @@ namespace Dialog
         assert( _window );
         return _window->totalArea();
     }
+
+    void ItemSelectionWindow::enableToggleButtons()
+    {
+        _buttonToggleOn.enable();
+        _buttonToggleOff.enable();
+
+        const bool isEvilInterface = Settings::Get().isEvilInterfaceEnabled();
+
+        _window->renderButton( _buttonToggleOn, isEvilInterface ? ICN::BUTTON_TOGGLE_ALL_ON_EVIL : ICN::BUTTON_TOGGLE_ALL_ON_GOOD, 0, 1, { 0, 7 },
+                               fheroes2::StandardWindow::Padding::BOTTOM_CENTER );
+        _buttonToggleOn.disable();
+
+        _window->renderButton( _buttonToggleOff, isEvilInterface ? ICN::BUTTON_TOGGLE_ALL_OFF_EVIL : ICN::BUTTON_TOGGLE_ALL_OFF_GOOD, 0, 1, { 0, 7 },
+                               fheroes2::StandardWindow::Padding::BOTTOM_CENTER );
+    }
 }
 
 Skill::Secondary Dialog::selectSecondarySkill( const Heroes & hero, const int skillId /* = Skill::Secondary::UNKNOWN */ )
@@ -922,6 +997,11 @@ Skill::Secondary Dialog::selectSecondarySkill( const Heroes & hero, const int sk
         }
     }
 
+    if ( skills.empty() ) {
+        fheroes2::showStandardTextMessage( _( "Warning" ), _( "There are no secondary skills to select from." ), Dialog::OK );
+        return {};
+    }
+
     SelectEnumSecSkill listbox( { 350, fheroes2::Display::instance().height() - dialogHeightDeduction }, _( "Select Skill:" ) );
 
     listbox.SetListContent( skills );
@@ -931,7 +1011,7 @@ Skill::Secondary Dialog::selectSecondarySkill( const Heroes & hero, const int sk
 
     const int32_t result = listbox.selectItemsEventProcessing();
 
-    if ( result == Dialog::OK ) {
+    if ( result == Dialog::OK && listbox.IsValid() ) {
         const int skillIndex = listbox.GetCurrent();
         return { SelectEnumSecSkill::getSkillFromListIndex( skillIndex ), SelectEnumSecSkill::getLevelFromListIndex( skillIndex ) };
     }
@@ -956,6 +1036,11 @@ Spell Dialog::selectSpell( const int spellId, const bool includeRandomSpells, co
         }
     }
 
+    if ( spells.empty() ) {
+        fheroes2::showStandardTextMessage( _( "Warning" ), _( "There are no spells to select." ), Dialog::OK );
+        return { Spell::NONE };
+    }
+
     SelectEnumSpell listbox( { 340, fheroes2::Display::instance().height() - dialogHeightDeduction }, _( "Select Spell:" ) );
 
     listbox.SetListContent( spells );
@@ -965,7 +1050,7 @@ Spell Dialog::selectSpell( const int spellId, const bool includeRandomSpells, co
 
     const int32_t result = listbox.selectItemsEventProcessing();
 
-    return ( result == Dialog::OK ) ? Spell( listbox.GetCurrent() ) : Spell( Spell::NONE );
+    return { ( result == Dialog::OK && listbox.IsValid() ) ? listbox.GetCurrent() : Spell::NONE };
 }
 
 Artifact Dialog::selectArtifact( const int artifactId, const bool isForVictoryConditions )
@@ -1005,6 +1090,11 @@ Artifact Dialog::selectArtifact( const int artifactId, const bool isForVictoryCo
         artifacts.emplace_back( id );
     }
 
+    if ( artifacts.empty() ) {
+        fheroes2::showStandardTextMessage( _( "Warning" ), _( "There are no artifacts to select from." ), Dialog::OK );
+        return { Artifact::UNKNOWN };
+    }
+
     SelectEnumArtifact listbox( { 370, fheroes2::Display::instance().height() - dialogHeightDeduction }, _( "Select Artifact:" ) );
 
     listbox.SetListContent( artifacts );
@@ -1014,7 +1104,7 @@ Artifact Dialog::selectArtifact( const int artifactId, const bool isForVictoryCo
 
     const int32_t result = listbox.selectItemsEventProcessing();
 
-    return ( result == Dialog::OK ) ? Artifact( listbox.GetCurrent() ) : Artifact( Artifact::UNKNOWN );
+    return { ( result == Dialog::OK && listbox.IsValid() ) ? listbox.GetCurrent() : Artifact::UNKNOWN };
 }
 
 Monster Dialog::selectMonster( const int monsterId )
@@ -1025,6 +1115,11 @@ Monster Dialog::selectMonster( const int monsterId )
     std::iota( monsters.begin(), monsters.end(), Monster::UNKNOWN + 1 );
     monsters.erase( std::remove_if( monsters.begin(), monsters.end(), []( const int id ) { return Monster( id ).isRandomMonster(); } ), monsters.end() );
 
+    if ( monsters.empty() ) {
+        fheroes2::showStandardTextMessage( _( "Warning" ), _( "There are no monsters to select from." ), Dialog::OK );
+        return { Monster::UNKNOWN };
+    }
+
     SelectEnumMonster listbox( { 320, fheroes2::Display::instance().height() - dialogHeightDeduction }, _( "Select Monster:" ) );
 
     listbox.SetListContent( monsters );
@@ -1034,7 +1129,7 @@ Monster Dialog::selectMonster( const int monsterId )
 
     const int32_t result = listbox.selectItemsEventProcessing();
 
-    return ( result == Dialog::OK ) ? Monster( listbox.GetCurrent() ) : Monster( Monster::UNKNOWN );
+    return { ( result == Dialog::OK && listbox.IsValid() ) ? listbox.GetCurrent() : Monster::UNKNOWN };
 }
 
 int Dialog::selectHeroes( const int heroId /* = Heroes::UNKNOWN */ )
@@ -1044,6 +1139,11 @@ int Dialog::selectHeroes( const int heroId /* = Heroes::UNKNOWN */ )
 
     std::vector<int> heroes( static_cast<int>( isPoLHeroesAllowed ? Heroes::JARKONAS : Heroes::BRAX ), Heroes::UNKNOWN );
 
+    if ( heroes.empty() ) {
+        fheroes2::showStandardTextMessage( _( "Warning" ), _( "There are no heroes to select from." ), Dialog::OK );
+        return Heroes::UNKNOWN;
+    }
+
     std::iota( heroes.begin(), heroes.end(), Heroes::UNKNOWN + 1 );
 
     SelectEnumHeroes listbox( { 300, fheroes2::Display::instance().height() - dialogHeightDeduction }, _( "Select Hero:" ) );
@@ -1055,12 +1155,12 @@ int Dialog::selectHeroes( const int heroId /* = Heroes::UNKNOWN */ )
 
     const int32_t result = listbox.selectItemsEventProcessing();
 
-    return ( result == Dialog::OK ) ? listbox.GetCurrent() : Heroes::UNKNOWN;
+    return ( result == Dialog::OK && listbox.IsValid() ) ? listbox.GetCurrent() : Heroes::UNKNOWN;
 }
 
 void Dialog::multiSelectMonsters( std::vector<int> allowed, std::vector<int> & selected )
 {
-    MultiMonsterSelection monsterList( { 320, fheroes2::Display::instance().height() - dialogHeightDeduction }, _( "Select Monsters:" ) );
+    MultiMonsterSelection monsterList( { 380, fheroes2::Display::instance().height() - dialogHeightDeduction }, _( "Select Monsters:" ) );
     monsterList.setup( std::move( allowed ), selected );
 
     const int32_t result = monsterList.selectItemsEventProcessing();
