@@ -92,7 +92,7 @@ private:
 
     void _renderScenarioListItem( const Maps::FileInfo & info, fheroes2::Display & display, const int32_t dsty, const bool current ) const;
     void _renderSelectedScenarioInfo( fheroes2::Display & display, const fheroes2::Point & dst );
-    void _renderMapName( const Maps::FileInfo & info, bool selected, const int32_t & baseYOffset, fheroes2::Display & display ) const;
+    void _renderMapName( const Maps::FileInfo & info, bool selected, const int32_t baseYOffset, fheroes2::Display & display ) const;
     static void _renderMapIcon( const uint16_t size, fheroes2::Display & display, const int32_t coordX, const int32_t coordY );
     static const fheroes2::Sprite & _getPlayersCountIcon( const PlayerColorsSet colors );
     static const fheroes2::Sprite & _getMapTypeIcon( const GameVersion version );
@@ -102,5 +102,5 @@ private:
 
 namespace Dialog
 {
-    const Maps::FileInfo * SelectScenario( const MapsFileInfoList & allMaps, const bool isForEditor );
+    const Maps::FileInfo * SelectScenario( MapsFileInfoList & all, const bool isForEditor );
 }
